{Version 2.05 19/8/98}
unit Jreknob;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls;

type
  TJREKnob = class(TCustomPanel)
  private
    Cx, Cy, Radius: integer;
    OldKnobLine: TPoint;
    FKnobColour: TColor;
    FMaxValue: integer;
    FMinValue: integer;
    FValue: integer;
    FScale: integer;
    FOnChange: TNotifyEvent;
    FEnabled: boolean;
  protected
    CapLabel: TLabel;
    EdgeSize: integer;
    PaintBottom: integer;
    ButtonDown: boolean;
    ButtonNow: TMouseButton;
    procedure Paint; override;
    procedure SetName(const NewName: TComponentName); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DoChange; virtual;
    procedure PaintKnob;
    procedure PaintScale;
    procedure Resize; override;
    procedure CreateWnd; override;
    procedure SetMaxValue(NewVal: integer);
    procedure SetMinValue(NewVal: integer);
    procedure SetValue(NewVal: integer); virtual;
    procedure SetScale(NewVal: integer);
    procedure SetEnabled(NewVal: boolean); virtual;
    procedure SetKnobColour(NewVal: TColor);
    function  GetCaption: TCaption;
    procedure SetCaption(NewVal: TCaption);
    function  GetCaptionAlign: TAlign;
    procedure SetCaptionAlign(NewVal: TAlign);
    property  BevelOuter default bvNone;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property  Width default 87;
    property  Height default 119;
    property  KnobColour: TColor read FKnobColour write SetKnobColour default clGray;
    property  MaxValue: integer read FMaxValue write SetMaxValue default 127;
    property  MinValue: integer read FMinValue write SetMinValue default -127;
    property  Value: integer read FValue write SetValue default 0;
    property  Scale: integer read FScale write SetScale default 8;
      {Scale range arbitrarily set 3..20}
    property  Enabled: boolean read FEnabled write SetEnabled default true;
    property  Caption: TCaption read GetCaption write SetCaption;
    property  CaptionAlign: TAlign read GetCaptionAlign write SetCaptionAlign default alBottom;
      {Only alTop or alBottom allowed}
    property  OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TJREKnob]);
end;

constructor TJREKnob.Create(AOwner: TComponent);
begin
  try
    inherited Create(AOwner);
    Width := 87;
    Height := 119;
    BevelOuter := bvNone;
    FKnobColour := clGray;
    FMaxValue := 127;{ Always make sure default Max > Min}
    FMinValue := -127;{ to avoid divide-by-zero risk}
    FValue := 0;
    FScale := 8;
    FEnabled := true;
    ButtonDown := false;
    CapLabel := TLabel.Create(self);
    CapLabel.Parent := self;
    CapLabel.Caption := 'JustCreated';
    CapLabel.Align := alBottom;
    CapLabel.WordWrap := true;
    CapLabel.Alignment := taCenter;
  except
    MessageBox(0, 'Unable to create knob: closing down program', 'JREKnob.Create function', mb_OK);
    Application.Terminate;
  end;
end;

procedure TJREKnob.CreateWnd;
begin
  Resize;
  inherited CreateWnd;
end;

procedure TJREKnob.Resize;
begin
  if BorderStyle = bsNone then EdgeSize := 0 else EdgeSize := 1;
  if BevelOuter <> bvNone then EdgeSize := EdgeSize + BevelWidth;
  if BevelInner <> bvNone then EdgeSize := EdgeSize + BevelWidth + BorderWidth;
    {These are not published in TJREKnob but may be in descendents}
  Cx := Width div 2;
  Cy := Cx;
  if CapLabel.Align = alTop then Inc(Cy, CapLabel.Height);
  Radius := Round((Cx - EdgeSize) * 0.7);
  PaintBottom := Cy + Cx - EdgeSize;
end;

procedure TJREKnob.PaintScale;
var
  angle: double;
  step: integer;
  Outside : integer;
begin
  {Resize;}
  Outside := Round(Radius * 1.25);
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Width := 1;
  for step := 0 to FScale do begin
    angle := -2.5 + step * 5 / FScale - pi / 2;
    Canvas.MoveTo(Cx + Round(Outside * Cos(angle)), Cy + Round(Outside * Sin(angle)));
    Canvas.LineTo(Cx + Round((Radius + 2) * Cos(angle)), Cy + Round((Radius + 2) * Sin(angle)));
  end;
end;

procedure TJREKnob.Paint;
begin
  CapLabel.AutoSize := false;
  CapLabel.AutoSize := true;
  inherited Paint;
  PaintScale;
  PaintKnob;
end;

procedure TJREKnob.PaintKnob;
var
  Intermed: double;
begin
  {Resize;}
  if Enabled then
    Canvas.Brush.Color := KnobColour
  else Canvas.Brush.Color := clSilver;
  if KnobColour = clBlack then
    Canvas.Pen.Color := clWhite
  else Canvas.Pen.Color := clBlack;
  Canvas.Pen.Width := 1;
  Canvas.Ellipse(Cx - Radius, Cy - Radius, Cx + Radius, Cy + Radius);
  Canvas.MoveTo(Cx, Cy);
  Canvas.Pen.Width := 2;
  Intermed := (Value * 2 - MaxValue - MinValue) * 2.5;
  OldKnobLine.X := Cx + Round((Radius - 2) * Sin(Intermed / (MaxValue - MinValue)));
  OldKnobLine.Y := Cy - Round((Radius - 2) * Cos(Intermed / (MaxValue - MinValue)));
  Canvas.LineTo(OldKnobLine.X, OldKnobLine.Y);
end;

procedure TJREKnob.SetName(const NewName: TComponentName);
begin
  if (CapLabel <> nil) and ((CapLabel.Caption = 'JustCreated')
    or (CapLabel.Caption = Name)) then CapLabel.Caption := NewName;
  inherited SetName(NewName);
  inherited Caption := '';
end;

procedure TJREKnob.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  angle: double;
  OldValue: integer;
begin
  if Enabled and (Y < (Cx + Cy))then begin
    OldValue := Value;
    ButtonDown := true;
    ButtonNow := Button;
    if Y = Cy then begin
      if X > Cx then angle := pi / 2 else angle := -pi / 2;
    end else
    if Y < Cy then
      angle := ArcTan((Cx - X)/(Y - Cy + 0.0001))
    else if X > Cx then
    angle := pi - ArcTan((X - Cx)/(Y - Cy + 0.0001))
    else angle := -ArcTan((X - Cx)/(Y - Cy + 0.0001)) - pi;
    Value := Round(angle * (MaxValue - MinValue) / 5 + (MinValue + MaxValue) / 2);
    if Value > MaxValue then Value := MaxValue;
    if Value < MinValue then Value := MinValue;
    if Value <> OldValue then begin
      PaintKnob;
      DoChange;
    end;
  end;
end;

procedure TJREKnob.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if ButtonDown then
    if (X > 0) and (Y > (Cy - Cx)) and (X < Width) and (Y < (Cy + Cx)) then
      MouseDown(ButtonNow, Shift, X, Y);
end;

procedure TJREKnob.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  ButtonDown := false;
end;

procedure TJREKnob.DoChange;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TJREKnob.SetMaxValue(NewVal: integer);
begin
  if NewVal <> MaxValue then begin
    if NewVal > MinValue then FMaxValue := NewVal
    else FMaxValue := MinValue + 1;
    if Value > MaxValue then begin
      Value := MaxValue;
      DoChange;
    end;
    Invalidate;
  end;
end;

procedure TJREKnob.SetMinValue(NewVal: integer);
begin
  if NewVal <> MinValue then begin
    if NewVal < MaxValue then FMinValue := NewVal
    else FMinValue := MaxValue - 1;
    if Value < MinValue then begin
      Value := MinValue;
      DoChange;
    end;
    Invalidate;
  end;
end;

procedure TJREKnob.SetValue(NewVal: integer);
begin
  if NewVal <> Value then begin
    if NewVal > FMaxValue then FValue := FMaxValue
    else if NewVal < FMinValue then FValue := FMinValue
    else FValue := NewVal;
    PaintKnob;
    DoChange;
  end;
end;

procedure TJREKnob.SetScale(NewVal: integer);
begin
  if NewVal <> Scale then begin
    if NewVal < 3 then FScale := 3
    else if NewVal > 20 then FScale := 20
    else FScale := NewVal;
    Invalidate;
  end;
end;

procedure TJREKnob.SetEnabled(NewVal: boolean);
begin
  if NewVal <> Enabled then begin
    FEnabled := NewVal;
    CapLabel.Enabled := NewVal;
    Invalidate;
  end;
end;

procedure TJREKnob.SetKnobColour(NewVal: TColor);
begin
  if NewVal <> KnobColour then begin
    FKnobColour := NewVal;
    Invalidate;
  end;
end;

function  TJREKnob.GetCaption: TCaption;
begin
  Result := CapLabel.Caption;
end;


procedure TJREKnob.SetCaption(NewVal: TCaption);
begin
  if CapLabel.Caption <> NewVal then begin
    CapLabel.Caption := NewVal;
    Invalidate;
  end;
end;

function TJREKnob.GetCaptionAlign: TAlign;
begin
  Result := CapLabel.Align;
end;

procedure TJREKnob.SetCaptionAlign(NewVal: TAlign);
begin
  if (NewVal <> CaptionAlign) and ((NewVal = alTop) or (NewVal = alBottom)) then begin
    CapLabel.Align := NewVal;
    Resize;
    Invalidate;
  end;
end;

end.

{Version history
2.04 First released version: DSP 1/8/98
2.05 SetValue and SetEnable made virtual
Port to Delphi3 23/8/98
}
